* This program computes mean and/or median WTP and Krinsky and Robb procedure confidence intervals, 
* achieved significance levels, and relative efficiency measures.
*! Author PWJ
* March 2007
* Version 1.0
program wtpcikr, rclass
	version 9.2
	if (`"`e(cmd)'"' != "probit" & `"`e(cmd)'"' != "logit" & `"`e(cmd)'"' != "biprobit" & `"`e(cmd)'"' != "xtprobit") {
		di in y "You are responsable as to whether WTP formulas apply for the command `e(cmd)'"		
	}
	global est_cmd "`e(cmd)'"
	syntax varlist [if] [in] [, REPS(integer 5000) SEED(integer 032007) Level(integer 95) MYMean(name) BMat(name) VMat(name) ///
	EQuation(string) EXPOnential MEANList DOTS]				   
	marksample touse
	tempname v b c upay model ul ll mean
	tempfile simdat
	_estimates hold `model', copy
	if `"`bmat'"'!="" mat `b'=`bmat'
	else mat `b'=e(b)
	if `"`vmat'"' !="" mat `v'=`vmat'
	else	mat `v'=e(V)	
	
	/* If there are multiple equations as in bivariate probit or commands or programs which estimate	
 	ancillary parameters, use the first one, unless equation is specified */
	if "`equation'"=="" { 
		loc nv: word count `varlist' 
		mat `b'=`b'[1,1..`nv'+1]
		mat `v'=`v'[1..`nv'+1,1..`nv'+1]
  	}	
	if "`equation'" != "" {
            mat `v' = `v'["`equation':","`equation':"]  
            mat rownames `v' = :
            mat colnames `v' = :
            mat `b' = `b'[1,"`equation':"] 
            mat colnames `b' = :
      }
	loc k = colsof(`b')
	if `k'==2 & (`"`mymean'"'!="" | `"`meanlist'"'!="") {
		di as err "Option MYMEAN or MEANLIST not allowed in a constant-only model"
		exit 198
	}
	if `"`mymean'"' !="" & `"`meanlist'"' !=""  {
		di as err "Options MYMEAN and  MEANLIST may not be combined."
		exit 184
	}
	wtpmeasure `varlist' if `touse', bvec(`b') model($est_cmd) mym(`mymean') `exponential' `meanlist' 
	if `"`meanlist'"' !="" matrix meanlist=r(meanvar) // will disappear if not captured here 	
	if `"`exponential'"' =="" local mean=r(meanwtp)  
	else {
		local mean=r(meanwtp)
		local median=r(medianwtp)
	}
	preserve
	if `"`dots'"' !="" local showdots ""
	else local showdots "nodots"
	set seed `seed' // To ensure replicability of results
	simulate, reps(`reps') `showdots' nol saving(`simdat'): wtpkrinsky `varlist', b(`b') v(`v') mod($est_cmd) mym(`mymean') ///
	`exponential'
	if `"`exponential'"'=="" {	
		* Achieved significant level (ASL)
		gen indicator= (meanwtp<=0) // Since the alternative hypothesis is meanwtp>0
		sum indicator, meanonly
		local asl=r(mean)
		* Now confidence interval
		sort meanwtp
		scalar `c'=((100-`level')/2)/100
		qui drop if _n<=round(`c'*`reps')  // dropping the top 2.5%
		qui drop if _n>=_N-round(`c'*`reps')+1 // dropping the bottom 2.5%
		local ll=meanwtp[1]
		local ul=meanwtp[_N]
		local cimean=(`ul'-`ll')/`mean'
		matrix `upay'=J(1,5,0)
		local i=1
		local val `mean' `ll' `ul' `asl' `cimean'
		foreach s of local val {
			matrix `upay'[1,`i']=`s'
			local i=`i'+1
		}
		matrix colnames `upay' = WTP LB UB ASL* CI/MEAN
		matrix rownames `upay' = MEAN/MEDIAN
		di
		matlist `upay', title(Krinsky and Robb Confidence Interval for WTP measures) border(rows) ///
		rowtitle(MEASURE) cspec(| %12s | %8.2f | %12.2f | %12.2f | %8.4f | %8.2f |) rspec(---)
		di as txt "*: Achieved Significance Level for testing H0: WTP<=0 vs. H1: WTP>0"
		di as txt "LB: Lower bound; UB: Upper bound"
	}
	else {
		local measure mean median
		qui {
			foreach var of local measure {
				use `simdat', clear						
				if `var'==. gen ind`var'=.
				else gen ind`var'= (`var'<=0)
				sum ind`var', meanonly
				loc asl`var'=r(mean)							
				sort `var'
				sca `c'=((100-`level')/2)/100
				drop if _n<=round(`c'*`reps')  // dropping the top 2.5%
				drop if _n>=_N-round(`c'*`reps')+1 // dropping the bottom 2.5%
				local ll`var'=`var'[1]
				local ul`var'=`var'[_N]				
				local ci`var'=(`ul`var''-`ll`var'')/``var'' // note double quoting
				local al`var' ``var'' `ll`var'' `ul`var'' `asl`var'' `ci`var'' // ``var'' contains the locals `mean' and `median'
			}
		}
		matrix `upay'=J(2,5,0)
		local i=1
		local j=1
		foreach s of local almean {
			matrix `upay'[1,`i']=`s'
			local i=`i'+1
		}
		foreach r of local almedian {
			matrix `upay'[2,`j']=`r'
			local j=`j'+1
		}
		matrix colnames `upay' = WTP LB UB ASL* CI/MEAN
		matrix rownames `upay' = MEAN MEDIAN		
		di
		matlist `upay', title(Krinsky and Robb Confidence Interval for WTP measures) rowtitle(MEASURES) ///
		cspec(| %12s | %8.2f | %12.2f | %12.2f | %8.4f | %8.2f |) rspec(----)
		di as txt "*: Achieved Significance Level for testing H0: WTP<=0 vs. H1: WTP>0"
		di as txt "LB: Lower bound; UB: Upper bound"
		
	}
	if `"`meanlist'"'!="" {
		gettoken bidv xvars: varlist
		matrix colnames meanlist = `xvars'
		matrix rownames meanlist = Mean
		di
		matlist meanlist, for(%8.5f) border(rows) title(Sample mean of the variables used in the computation)
	}	
	restore
     _estimates unhold `model' 
	 
end

